{% load menu_tags %}
{% if menu_items %}
	<ul class="dropdown-menu" aria-labelledby="ddtoggle_{{ parent_page.pk }}">
	{% for item in menu_items %}
	    <li class="{{ item.active_class }}{% if item.has_children_in_menu %} dropdown{% endif %}">
	        <a href="{{ item.href }}"{% if item.has_children_in_menu %} class="dropdown-toggle" id="ddtoggle_{{ item.pk }}" data-toggle="dropdown" data-hover="dropdown" data-delay="400" data-close-others="false" aria-haspopup="true" aria-expanded="false"{% endif %}>{{ item.text }}</a>
	        {% if item.has_children_in_menu %}
	            {% sub_menu item template=current_template %}
	        {% endif %}
	    </li>
	{% endfor %}
	</ul>
{% endif %}
