{% load menu_tags %}
{% if menu_items %}
<nav class="nav-section" role="navigation">
    <ul>
        {% if show_section_root and section_root %}
        <li class="section_root">
            <a href="{{ section_root.href }}" class="{{ section_root.active_class }}">{{ section_root.text }}</a>
            <ul>
        {% endif %}
        {% for item in menu_items %}
        <li class="{{ item.active_class }}">
            <a href="{{ item.href }}">{{ item.text }}</a>
            {% if item.has_children_in_menu %}
                 {% sub_menu item %}
            {% endif %}
        </li>
        {% endfor %}
        {% if show_section_root and section_root %}
            </ul>
        </li>
        {% endif %}
    </ul>
</nav>
{% endif %}
