# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-12-04 21:43
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailmenus', '0018_auto_20161204_2043'),
    ]

    operations = [
        migrations.AlterField(
            model_name='flatmenu',
            name='use_specific',
            field=models.PositiveSmallIntegerField(choices=[(0, 'Off (most efficient)'), (1, 'Auto'), (2, 'Top level'), (3, 'Always (least efficient)')], default=1, help_text="Controls how 'specific' pages objects are fetched and used when rendering this menu. This value can be overidden by supplying a different <code>use_specific</code> value to the <code>{% flat_menu %}</code> tag in your templates.", verbose_name='specific page usage'),
        ),
        migrations.AlterField(
            model_name='mainmenu',
            name='use_specific',
            field=models.PositiveSmallIntegerField(choices=[(0, 'Off (most efficient)'), (1, 'Auto'), (2, 'Top level'), (3, 'Always (least efficient)')], default=1, help_text="Controls how 'specific' pages objects are fetched and used when rendering this menu. This value can be overidden by supplying a different <code>use_specific</code> value to the <code>{% main_menu %}</code> tag in your templates.", verbose_name='specific page usage'),
        ),
    ]
