# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-30 09:02
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0012_linkpage'),
    ]

    operations = [
        migrations.AlterField(
            model_name='linkpage',
            name='extra_classes',
            field=models.CharField(blank=True, help_text='Optionally specify css classes to be added to this page when it appears in menus.', max_length=100, verbose_name='menu item css classes'),
        ),
        migrations.AlterField(
            model_name='linkpage',
            name='url_append',
            field=models.CharField(blank=True, help_text='Use this to optionally append a #hash or querystring to the URL.', max_length=255, verbose_name='append to URL'),
        ),
    ]
