# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-12-10 00:04
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailmenus', '0019_auto_20161204_2143'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='flatmenuitem',
            options={},
        ),
        migrations.AlterModelOptions(
            name='mainmenuitem',
            options={},
        ),
        migrations.AlterField(
            model_name='flatmenu',
            name='site',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='wagtailcore.Site', verbose_name='site'),
        ),
        migrations.AlterField(
            model_name='flatmenuitem',
            name='link_text',
            field=models.CharField(blank=True, help_text="Provide the text to use for a custom URL, or set on an internal page link to use instead of the page's title.", max_length=255, verbose_name='link text'),
        ),
        migrations.AlterField(
            model_name='mainmenu',
            name='site',
            field=models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='wagtailcore.Site', verbose_name='site'),
        ),
        migrations.AlterField(
            model_name='mainmenuitem',
            name='link_text',
            field=models.CharField(blank=True, help_text="Provide the text to use for a custom URL, or set on an internal page link to use instead of the page's title.", max_length=255, verbose_name='link text'),
        ),
    ]
