# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2017-01-06 23:55
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0010_auto_20161211_2116'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='customflatmenuitem',
            options={'ordering': ('sort_order',)},
        ),
        migrations.AlterModelOptions(
            name='custommainmenuitem',
            options={'ordering': ('sort_order',)},
        ),
        migrations.AlterModelOptions(
            name='flatmenucustommenuitem',
            options={'ordering': ('sort_order',)},
        ),
        migrations.AlterModelOptions(
            name='mainmenucustommenuitem',
            options={'ordering': ('sort_order',)},
        ),
    ]
