{% load menu_tags %}
{% if menu_items %}
    {% if show_section_root and section_root %}
        <a href="{{ section_root.href }}" class="{{ section_root.active_class }} section_root{% if section_root.extra_menuitem_css_class %} {{ section_root.extra_menuitem_css_class }}{% endif %}">{{ section_root.text }}</a>
    {% endif %}
    <ul>
        {% for item in menu_items %}
        <li class="{{ item.active_class }}{% if item.extra_menuitem_css_class %} {{ item.extra_menuitem_css_class }}{% endif %}">
            <a href="{{ item.href }}">{{ item.text }}</a>
            {% if item.has_children_in_menu %}
                 {% sub_menu item %}
            {% endif %}
        </li>
        {% endfor %}
    </ul>
{% endif %}
