from __future__ import absolute_import, unicode_literals


class RemovedInWagtailMenus24Warning(DeprecationWarning):
    pass


removed_in_next_version_warning = RemovedInWagtailMenus24Warning


class RemovedInWagtailMenus25Warning(PendingDeprecationWarning):
    pass
