{% load menu_tags %}
<ul class="nav navbar-nav">
{% for item in menu_items %}
    <li class="{{ item.active_class }}{% if item.has_children_in_menu %} dropdown{% endif %}{% if item.link_page and item.link_page.extra_menuitem_css_class %} {{ item.link_page.extra_menuitem_css_class }}{% endif %}">
        <a href="{{ item.href }}"{% if item.has_children_in_menu %} class="dropdown-toggle" id="ddtoggle_{{ item.link_page.pk }}" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false"{% endif %}>{{ item.text }}{% if item.has_children_in_menu %} <span class="caret"></span>{% endif %}</a>
        {% if item.has_children_in_menu %}
        	{% sub_menu item %}
        {% endif %}
    </li>       
{% endfor %}
</ul>
